'''A simple logger'''

import os
import datetime
import uuid
from typing import Optional

def gen_uuid() -> str:
    '''Generates a UUID'''
    return str(uuid.uuid4())

class LoggerError(Exception):
    '''Exception raised for log file related errors'''

class Logger:
    '''A simple logger class'''

    cache: list[str] = None

    def __init__(
                self,
                file_path: Optional[str] = None,
                encoding: str = "utf-8-sig",
                time_format: str = "%Y.%m.%d %H:%M:%S",
                separator: str = "\t",
                silent: bool = False,
                enable_remove: bool = False,
                strip_content: bool = False
            ):
        '''
        Args:
            file_path: path of the log file
            encoding: encoding of the log file
            time_format: time format
            separator: separator
            silent: if True, it will not print the log, just write it to the file
            enable_remove: if True, it will enable the remove function
            strip_content: if True, it will strip the content
        '''
        self.file_path = file_path if file_path else os.path.join(os.getcwd(), f"{gen_uuid()}.log")
        self.encoding = encoding
        self.time_format = time_format
        self.separator = separator
        self.__silent = silent
        self.__enable_remove = enable_remove
        self.__strip_content = strip_content

    def __str__(self) -> str:
        return f"Log file: {self.file_path}"

    def __error(self, message: str = "") -> None:
        '''Prints error message'''
        raise LoggerError(str(message))

    def __str_time(self) -> str:
        '''Returns the current time as a string'''
        current_time = datetime.datetime.now()
        try:
            return current_time.strftime(self.time_format)
        except Exception as e: # pylint: disable=broad-except
            print(f"Error: {e}")
            return current_time.strftime("%Y.%m.%d %H:%M:%S")

    def __log_to_file(self, content: str) -> None:
        '''Appends file'''
        with open(self.file_path, "a", encoding = self.encoding) as file:
            file.write(content)

    def __strip(self, content: str) -> str:
        '''Strips the content'''
        return content.strip() if self.__strip_content else content

    def log(self, content: str = "") -> str:
        '''Logs content to file'''
        content = self.__str_time() + self.separator + str(self.__strip(content)) + "\n"
        if not self.__silent:
            print(content.strip())
        self.__log_to_file(content)
        return content

    def change_path(self, file_path: str) -> str:
        '''Changes the log file path'''
        self.file_path = file_path
        print(f"Changed path from {self.file_path} to {file_path}")

    def change_encoding(self, encoding: str) -> str:
        '''Changes the log file encoding'''
        self.encoding = encoding
        print(f"Changed encoding to {encoding}")

    def change_time_format(self, time_format: str) -> str:
        '''Changes the time format'''
        self.time_format = time_format
        print(f"Changed time format to {time_format}")

    def change_separator(self, separator: str) -> str:
        '''Changes the separator'''
        self.separator = separator
        print(f"Changed separator to {separator}")

    def clear(self) -> None:
        '''Clears the log file'''
        if self.__enable_remove:
            if os.path.exists(self.file_path):
                with open(self.file_path, "w", encoding = self.encoding) as _:
                    print(f"Log file cleared ({self.file_path})")
            self.__error(f"Log file does not exist ({self.file_path})")
        self.__error("Removal is not enabled")

    def remove(self) -> None:
        '''Removes the log file'''
        if self.__enable_remove:
            if os.path.exists(self.file_path):
                os.remove(self.file_path)
                print(f"Log file removed ({self.file_path})")
            self.__error(f"Log file does not exist ({self.file_path})")
        self.__error("Removal is not enabled")



'''Writing .xlsx files'''

import os
from datetime import date
from decimal import Decimal
import xlsxwriter

class WorkSheet:
    '''Worksheet class for the workbook class
    name: name of the worksheet
    header: header of the worksheet, list[str]
    data: data of the worksheet, list[list[]]
    header_comment: comment on the headers, list[[str, str]]'''

    def __init__(self, name: str, header: list[str], data: list[list], header_comment: list = None):
        self.name: str = name
        self.header: list[str] = header
        self.data: list[list] = data
        self.header_comment: list = header_comment if header_comment else []

    def __str__(self):
        return self.name

    # Seggesting a column width with min. and max. filter
    def suggest_width(self, col: int, min_width: int = 10, max_width: int = 40):
        '''Suggesting a column width with min. and max. filter'''

        length: int = min_width
        if self.header:
            if len(str(self.header[col])) > length:
                length: int = len(str(self.header[col]))
        if self.data:
            for row in self.data:
                if len(row) > col and len(str(row[col])) > length:
                    length: int = len(str(row[col]))

        return (length if length < max_width else max_width)


class WorkBook:
    '''Workbook class
    name: name of the workbook
    content: content of the workbook, worksheet or list[worksheet]'''

    def __init__(self, name: str, content: list[WorkSheet]):
        self.name: str = name
        self.content: list[WorkSheet] = content

    def __str__(self):
        return self.name
    
    # Creating the .xlsx
    def xlsx_create(self, file_path: str = os.getcwd(), file_name: str = None) -> str:
        '''Creating the .xlsx'''

        # Creating the name of the .xlsx
        file_name: str = (self.name if file_name is None else file_name)
        if not file_name.lower().endswith(".xlsx"):
            file_name += ".xlsx"
        excel_file: str = os.path.join(file_path, file_name)

        # Creating the .xlsx
        file: xlsxwriter.workbook = xlsxwriter.Workbook(excel_file)
        bold_format = file.add_format({"bold": True})
        date_format = file.add_format({'num_format': 'yyyy.mm.dd'})
        number_format = file.add_format({'num_format': '#,##0.00'})

        # Fetching the worksheet(s)
        worksheets: list = []
        if type(self.content) != list:
            worksheets.append(self.content)
        else:
            for element in self.content:
                worksheets.append(element)

        print(f"{file_name} is {str(len(worksheets))} worksheet(s):")

        # Creating the worksheet(s)
        for wsheet in worksheets:
            print(f"\t {wsheet.name}")
            sheet = file.add_worksheet(wsheet.name)
            row: int = 0
            col: int = 0
            last_row: int = row
            last_col: int = col

            # If available, then creating a header
            if wsheet.header:
                sheet.freeze_panes(1, 0)
                for element in wsheet.header:
                    width = wsheet.suggest_width(col)
                    sheet.set_column(col, col, width)
                    sheet.write(row, col, element, bold_format)
                    # If available, then adding comments
                    if wsheet.header_comment:
                        for comment in wsheet.header_comment:
                            if element == comment[0]:
                                sheet.write_comment(row, col, comment[1])
                    col += 1
                    last_col = (col if col > last_col else last_col)

            # If available, then creating the data
            if wsheet.data:
                row += 1
                last_row = (row if row > last_row else last_row)
                for line in wsheet.data:
                    col = 0
                    for element in line:
                        # Checking for basic types
                        if isinstance(element, date):
                            try:
                                sheet.write_datetime(row, col, element, date_format)
                            except:
                                pass
                        elif isinstance(element, Decimal):
                            try:
                                sheet.write(row, col, element, number_format)
                            except:
                                pass
                        else:
                            try:
                                sheet.write(row, col, element)
                            except:
                                pass
                        col += 1
                        last_col = (col if col > last_col else last_col)
                    row += 1
                    last_row = (row if row > last_row else last_row)

            # If header is available, then adding an aoutfilter
            if wsheet.header:
                if last_col != 0:
                    last_col -= 1
                sheet.autofilter(0, 0, last_row, last_col)

        # Closing the .xlsx
        file.close()

        print(f"xlsx created: {excel_file}")

        return excel_file



'''setup'''

from pathlib import Path
from setuptools import setup, find_packages

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name = "villog",
    version = "0.1.3",
    description = "A simple python library for logging",
    author = "Krisztián Villers",
    packages = find_packages(),
    long_description = long_description,
    long_description_content_type = "text/markdown",
    install_requires=[
          'xlsxwriter'
    ]
)
